"""A thin wrapper around numpy arrays to dynamically grow them.
   Limited to 1D arrays.
"""

from numpy import zeros, concatenate
#from scipy.sparse import lil_matrix, vstack

class EArray(object):
    def __init__(self,initsize=100):
        #self.memarray = lil_matrix((initsize,1))
        self.memarray = zeros(initsize)
        self.num_elem = 0
        
    def __setitem__(self, idx, item):
        """When setting a value, grow array if neccesary"""
        #if idx >= self.memarray.shape[0]:
        if idx >= len(self.memarray):
            self.grow_array()
        if idx > self.num_elem:
            self.num_elem = idx+1
        #self.memarray[idx,0] = item
        self.memarray[idx] = item

    def __getitem__(self, idx):
        """Getting an item assumes that idx < self.num_elem,
        and hence < len(self.memarray)"""
        #return self.memarray[idx,0]
        return self.memarray[idx]

    def __len__(self):
        return self.num_elem

    @property
    def array(self):
        """The actual array itself, with correct size"""
        #return self.memarray[:self.num_elem,0].todense()
        return self.memarray[:self.num_elem]

    def grow_array(self):
        """Double the length of the array"""
        #self.memarray = vstack([self.memarray, lil_matrix((self.memarray.shape[0],1))],
        #                       format='lil')
        self.memarray = concatenate([self.memarray, zeros(len(self.memarray))])

    def append(self, elem):
        """Append a single element to the end of the array"""
        #if self.num_elem >= self.memarray.shape[0]:
        if self.num_elem >= len(self.memarray):
            self.grow_array()
        self.memarray[self.num_elem] = elem
        self.num_elem += 1

if __name__ == '__main__':
    """Some code to check that things are running."""
    a = EArray()
    print 'Array has default length of: %d' % len(a)
    print 'Insert 1 at position 12'
    a[12] = 1
    print 'Append 2 to the end'
    a.append(2)
    print 'Insert 3 at position 99'
    a[99] = 3
    print 'Append 4 to the end'
    a.append(4)
    print 'Array has length: %d' % len(a)
    print 'Insert 5 at position 123'
    a[123] = 5
    print 'Array has length: %d' % len(a)
    print 'The array:'
    print a.array.T
    print 'Value at 12: %f' %a[12]
    print 'Value at 13: %f' %a[13]
    print 'Value at 99: %f' %a[99]
    print 'Value at 100: %f' %a[100]
    print 'Value at 123: %f' %a[123]
    
    
