from numpy import sign, nonzero, equal, zeros
from numpy import array, concatenate

def confusionMatrix(labels_test, labels_predicted):
    """Compute the matrix of predictions versus labels"""
    if len(labels_test) != len(labels_predicted):
        return 0
    TP = 0; FP = 0; TN = 0; FN = 0
    for i in range(0, len(labels_test)):
        if labels_test[i] == 0 or labels_predicted[i] == 0:
            return 0
        if labels_test[i] > 0:
            if labels_predicted[i] > 0: TP += 1
            else: FN +=1
        else:
            if labels_predicted[i] > 0: FP += 1
            else: TN += 1
    return (TP, TN, FP, FN)

def accuracy(output, labels_test):
    """How many correct predictions?"""
    if max(labels_test) > 1:
        return accuracy_multiclass(output, labels_test)
    else:
        TP, TN, FP, FN = confusionMatrix(labels_test, sign(output))
        return float(TP + TN) / (TP + TN + FP + FN)

def accuracy_multiclass(output, labels_test):
    """Multiclass accuracy"""
    int_out = map(int, output)
    int_lab = map(int, labels_test)
    return 1.0*len(nonzero(equal(int_out, int_lab))[0])/len(output)

def accuracy_mix(output, labels_test, mix_list):
    """Multiclass accuracy, but allowing class confusions,
    as defined by mix_list"""
    assert(output.shape == labels_test.shape)
    int_out = map(int, output)
    new_out = zeros(len(int_out), dtype=int)
    for cid in range(len(mix_list)):
        idx = array([], dtype=int)
        for mix in mix_list[cid]:
            idx = concatenate([idx, nonzero(equal(int_out, mix))[0]])
        new_out[idx] = cid

    int_lab = map(int, labels_test)
    new_lab = zeros(len(int_lab), dtype=int)
    for cid in range(len(mix_list)):
        idx = array([], dtype=int)
        for mix in mix_list[cid]:
            idx = concatenate([idx, nonzero(equal(int_lab, mix))[0]])
        new_lab[idx] = cid

    return 1.0*len(nonzero(equal(new_out, new_lab))[0])/len(output)
