#!/usr/bin/env python
import os
import numpy
from distutils.core import setup
from distutils.extension import Extension
from distutils.command.sdist import sdist
from Cython.Distutils import build_ext

class sdist_hg(sdist):
    """Append Mercurial repository version if 'dev' option is used with sdist"""
    user_options = sdist.user_options + [
            ('dev', None, "Add a dev marker")
            ]

    def initialize_options(self):
        sdist.initialize_options(self)
        self.dev = 0

    def run(self):
        if self.dev:
            suffix = '.dev%d' % self.get_tip_revision()
            self.distribution.metadata.version += suffix
        sdist.run(self)

    def get_tip_revision(self, path=os.path.expanduser('~')+'/Mercurial/OptWok'):
        from mercurial.hg import repository
        from mercurial.ui import ui
        from mercurial import node
        repo = repository(ui(), path)
        tip = repo.changelog.tip()
        return repo.changelog.rev(tip)


setup(name='OptWok',
      version='0.1',
      description='A mix of various different optimization methods.',
      long_description=open('README').read(),
      author='Cheng Soon Ong',
      author_email='chengsoon.ong@inf.ethz.ch',
      url='http://www.inf.ethz.ch/personal/cong/',
      packages=['optwok'],
      cmdclass = {'build_ext': build_ext,
                  'sdist': sdist_hg},
      ext_modules = [Extension('kernelbase', ['optwok/kernelbase.pyx'],
                               include_dirs = [numpy.get_include(),'.'])],
      license = 'GNU GPL version 3 or later',
    )

